/* 
 * File:   SpellChecker.h
 * Author: Will Matheson
 * Course: PROG 2400 (Data Structures), Section 700
 * Due:    March 25, 2013
 * 
 * Compares words in a document file to a dictionary.
 * Displays the document words not found in the dictionary as
 * "potentially misspelled words".
 * The dictionary is stored in a binary search tree (BST).
 *
 * Created on March 13, 2013, 3:06 PM
 */

#ifndef SPELLCHECKER_H
#define	SPELLCHECKER_H

#include <iostream>
#include <fstream>
#include "Tree.h"//BST
#include "Stack.h"//for document words to be checked
#include "LinkedList.h"//for misspelled words

class SpellChecker {
public:
    //big three
    //constructor (takes document and dictionary filenames)
    SpellChecker(std::string documentFileName, std::string dictionaryFileName);
    SpellChecker(const SpellChecker& orig);//copy constructor (not defined)
    virtual ~SpellChecker();//virtual destructor (not defined)
    
    //public utilities
    void checkSpelling();//check spelling in document
    std::string wordClean(std::string word);//"cleans" a word of special chars
    bool isNumericWord(std::string word);//returns if word is only digits
    std::string convertToLower(std::string word);//convert to lowercase
private:
    std::string _documentFileName;//location of document
    std::string _dictionaryFileName;//location of dictionary
    Tree* _dictionary;//BST for dictionary
    Stack* _documentWords;//stack of words to be checked against dictionary
    LinkedList* _misspelledWords;//store words that might be misspelled
    
    //internal functions
    bool loadDictionary();//load dictionary file into BST, ret true if success
    void displayDictionary();//display dictionary BST
    bool loadDocumentWords();//load document words to queue, ret true if success
    bool anyMisspelledWords();//search dict, ret true if any words misspelled
};

#endif	/* SPELLCHECKER_H */

