/* 
 * File:   Tree.cpp
 * Author: Will Matheson
 * Course: PROG 2400 (Data Structures), Section 700
 * Due:    March 25, 2013
 * 
 * Implementation for Tree (see header file for description)
 * 
 * Created on March 7, 2013, 5:29 PM
 */

#include "Tree.h"

//**BIG THREE**
//constructor
Tree::Tree() {
    _root = NULL;
    _levelOrder = false;
    _demoMode = false;
}

//copy constructor (not defined)
Tree::Tree(const Tree& orig) {
}

//virtual destructor (not defined)
Tree::~Tree() {
}

//**PUBLIC UTILITIES**
//insert a new node with a string (ignores duplicates)
void Tree::insert(std::string key) {
    //if we're just starting out, set up camp here
    if (_root == NULL)
    {
        _root = new TreeNode(key, NULL, NULL, NULL, false);            
    }    
    else //go find a place to insert (similar to searching)
    {
        plop(_root, key);    
    }
}

//rebalance tree
void Tree::rebalance(){
    //start with a pseudoroot to make things easier
    _pseudoRoot = new TreeNode();
    _pseudoRoot->setRight(_root);
    treeToVine();//converts tree to long vine
    if (_demoMode)
    {
        //converts vine to balanced tree, with stage-by-stage demo
        if (!_levelOrder)
        {
            _levelOrder = true;
            vineToTreeDemo();
            _levelOrder = false;
        }
        else
        {
            vineToTreeDemo();
        }
    }
    else
    {
        //converts vine to balanced tree
        vineToTree();
    }
    _root = _pseudoRoot->getRight();//root has probably changed
    delete _pseudoRoot;//no longer needed
    fixParentage();//fixes the back-pointers used to 'really' show the tree
}

//see if such-and-such a string is in the tree
bool Tree::found(std::string findMe) {
    return lookup(_root, findMe);
}

//set/cancel Level-Order print mode
void Tree::setLevelOrder(bool setting){
    _levelOrder = setting;
}

//set/cancel using demonstration versions of functions, when available
void Tree::setDemoMode(bool setting){
    _demoMode = setting;
}

//print the tree in order (In-Order default, Level-Order available with flag)
std::ostream& operator<<(std::ostream& output, Tree& tree){
    if (tree._levelOrder)
    {
        output << "The tree in level order, format Value[heritage,#children]:"
            << std::endl;
        tree.levelOrder(output);
    }
    else
    {
        output << "The tree in value order, one key per line:" << std::endl;
        tree.inOrder(tree._root, output);
    }
    output << std::endl;
    return output;
}

//fast integer log2, needed for vine-to-tree
//(what you need to do to 2 (2^?) to get a given value)
//http://stackoverflow.com/a/994647
unsigned int Tree::lg (unsigned int val) {
    unsigned int retVal = -1;//if given 0, but mathematical answer is infinity
    while (val != 0) {
        val >>= 1;//shift value to the right by one bit
        retVal++;//however many times that had to be done is what we want
        //since it represents the power of 2 to get the value
    }
    return retVal;
}

//**INTERNAL FUNCTIONS**
//recursive insert
void Tree::plop(TreeNode* node, std::string key) {
    //ignore attempts at duplicate inserts
    if (node->getKey() == key)
    {
        return;    
    }
    //can it go down the left side?
    if (key < node->getKey())
    {
        //there is no left side yet
        if (node->getLeft() == NULL)
        {
            node->setLeft(new TreeNode(key, NULL, NULL, node, false));                    
        }
        //there is a left side, so call this function again
        else
        {
            plop(node->getLeft(), key);
        }        
    }
    else //try the right side
    {
        //there is no right side yet
        if (node->getRight() == NULL)
        {
            node->setRight(new TreeNode(key, NULL, NULL, node, true));                    
        }
        //there is a right side, so call this function again
        else
        {
            plop(node->getRight(), key);
        }            
    }    
}

//recursive lookup
bool Tree::lookup(TreeNode* node, std::string findMe){
    //check if the (sub)tree is empty
    if (node == NULL)
    {
        return false;
    }
    //check here
    else if (node->getKey() == findMe)
    {
        //success!
        return true;    
    }
    //OK, we have to play a game of Telephone
    //first, check if it might be down the left subtree
    //this can be, provided the value sought is less than this node
    else if (findMe < node->getKey())
    {
        return lookup(node->getLeft(), findMe);
    }
    else //it might be down the right subtree
    {
        return lookup(node->getRight(), findMe);    
    }
}

//recursive In-Order traversal
//http://en.wikipedia.org/wiki/Tree_traversal#In-order
void Tree::inOrder(TreeNode* node, std::ostream& output){
    if (node == NULL)
    {
        return;    //okay, we're hitting the bottom
    }
    inOrder(node->getLeft(), output); //go left if you can (see above)
    output << node->getKey() << std::endl; //that'll do for our "visit"
    inOrder(node->getRight(), output); //go right if you can
}

//iterative Level-Order traversal
//http://leetcode.com/2010/09/printing-binary-tree-in-level-order.html
void Tree::levelOrder(std::ostream& output){
    if (_root == NULL)
    {
        output << "Empty tree." << std::endl;
        return;
    }
    int level = 0;//root is "0"
    output << level << ": ";
    int nodesInCurrentLevel = 0;
    int nodesInNextLevel = 0;
    int nodesOnLine = 0;
    int children;
    Queue* myLeafPile = new Queue();//store leaves in a first-in, first-out way
    myLeafPile->append(_root);//the root
    nodesInCurrentLevel++;
    while (!myLeafPile->isEmpty())
    {
        children = 0;
        TreeNode* node = myLeafPile->getDeleteFirst();
        nodesInCurrentLevel--;
        if (node != NULL)
        {
            if (nodesOnLine > 3)//max 4 nodes per line
            {
                output << std::endl << ".: ";
                nodesOnLine = 0;
            }
            output << node->getKey();
            nodesOnLine++;
            if (node->getLeft() != NULL)
            {
                children++;
                myLeafPile->append(node->getLeft());
                nodesInNextLevel++;
            }
            if (node->getRight() != NULL)
            {
                children++;
                myLeafPile->append(node->getRight());
                nodesInNextLevel++;
            }
            //show ancestry
            if (node->getParent() != NULL)
            {
                if (node->isRightSpawn())
                {
                    output << "[R-";
                }
                else
                {
                    output << "[L-";
                }
                output << node->getParent()->getKey() << ","
                        << children << "] ";
            }
            else
            {
                output << "[ROOT," << children << "]";
            }
        }
        if (nodesInCurrentLevel == 0) //go to next level
        {
            output << std::endl;
            level++;
            nodesOnLine = 0;
            if (nodesInNextLevel > 0)
            {
                output << level << ": ";
            }
            nodesInCurrentLevel = nodesInNextLevel;
            nodesInNextLevel = 0;        
        }
    }        
}

//"flattens" tree to vine of right-children only,
//allowing for subsequent conversion to balanced tree
void Tree::treeToVine(){
    //the end of the part that's been "vined"
    TreeNode* vineTail = _pseudoRoot;
    //the start of the part that might need more work (always vineTail's right)
    TreeNode* remainder = vineTail->getRight();
    //temp pointer to help with rotations
    TreeNode* temp;
    
    //keep track of number of nodes (will need for vine-to-tree)
    _size = 0;
    //while there's potential work left...
    while (remainder != NULL)
    {
        if (remainder->getLeft() == NULL)    
        {
            //we're done here, so move the vine tail down one
            vineTail = remainder;
            remainder = remainder->getRight();
            _size++;
        }
        else //rotate
        {
            temp = remainder->getLeft();
            remainder->setLeft(temp->getRight());
            temp->setRight(remainder);
            remainder = temp;
            vineTail->setRight(temp);//or remainder
        }
    }
}

//converts vine back to a balanced tree
void Tree::vineToTree(){
    _leafCount = _size + 1 - pow(2, lg(_size+1));  
    //scrunch (that number of) subtrees to the left
    compression(_leafCount);
    
    _size = _size - _leafCount;
    while (_size > 1)
    {
        //doing fewer and fewer cycles each time
        compression(_size / 2);
        _size = _size / 2;
    }
}

//converts vine back to a balanced tree (demo each stage)
void Tree::vineToTreeDemo(){
    _leafCount = _size + 1 - pow(2, lg(_size+1));  
    //scrunch (that number of) subtrees to the left
    std::cout << "Leaf count is " << _leafCount << std::endl;
    std::cout << "Compression " << _leafCount << " times:" << std::endl;
    compression(_leafCount);
    //output tree in level-order:
    _root = _pseudoRoot->getRight();
    fixParentage();
    std::cout << *this;
    std::cin.get();
    
    _size = _size - _leafCount;
    std::cout << "Size is " << _size << std::endl;
    while (_size > 1)
    {
        //doing fewer and fewer cycles each time
        compression(_size / 2);
        //output tree in level-order:
        _root = _pseudoRoot->getRight();
        fixParentage();
        std::cout << *this;
        std::cin.get();
        _size = _size / 2;
        if (_size > 1)
        {
            std::cout << "Compression " << _size << " times:" << std::endl;
        }
    }
}

//reposition subtrees to the left
void Tree::compression(int count){
    TreeNode* scanner;
    TreeNode* child;
    
    scanner = _pseudoRoot;
    for (int i=0; i < count; i++)
    {
        child = scanner->getRight();
        scanner->setRight(child->getRight());
        scanner = scanner->getRight();
        child->setRight(scanner->getLeft());
        scanner->setLeft(child);
    }
}

//Level-Order traversal just to fix "parentage" for display
//http://en.wikipedia.org/wiki/Tree_traversal#Breadth-first_2
void Tree::fixParentage(){
    Queue* myLeafPile = new Queue();
    myLeafPile->append(_root);
    //"fix" new root
    _root->setParent(NULL);
    while (!myLeafPile->isEmpty())
    {
        TreeNode* node = myLeafPile->getDeleteFirst();
        //do stuff with node
        if (node->getLeft() != NULL)
        {
            node->getLeft()->setParent(node);
            node->getLeft()->setRightSpawn(false);
            myLeafPile->append(node->getLeft());
        }
        if (node->getRight() != NULL)
        {
            node->getRight()->setParent(node);
            node->getRight()->setRightSpawn(true);
            myLeafPile->append(node->getRight());        
        }
    }    
}
