/* 
 * File:   Tree.h
 * Author: Will Matheson
 * Course: PROG 2400 (Data Structures), Section 700
 * Due:    March 25, 2013
 * 
 * A binary search tree made up of TreeNodes.
 * Can be viewed In-Order (in value order)
 *  or in Level-Order (level by level, left to right).
 * Can be rebalanced for optimal search performance.
 * 
 * Trees:
 * 
 * Mary K. Vernon (University of Wisconsin-Madison)
 * "Binary Search Trees"
 * CS 367: Introduction to Data Structures
 * http://pages.cs.wisc.edu/~vernon/cs367/notes/9.BST.html
 * 
 * Tree Rebalancing:
 * 
 * Quentin F. Stout (University of Michigan)
 *  and Bette L. Warren (Eastern Michigan University)
 * "Tree Rebalancing in Optimal Time and Space"
 * Communications of the ACM, September 1986, vl.29 no.9
 * http://web.eecs.umich.edu/~qstout/abs/CACM86.html
 * 
 * See also:
 * 
 * Timothy J. Rolfe (Eastern Washington University)
 * "One-Time Binary Search Tree Balancing: The Day/Stout/Warren (DSW) Algorithm"
 * ACM SIGCSE Bulletin, December 2002, vl.34, is.4
 * http://penguin.ewu.edu/~trolfe/DSWpaper/
 *
 * Created on March 7, 2013, 5:29 PM
 */

#ifndef TREE_H
#define	TREE_H

#include <iostream>
#include <math.h>
#include "TreeNode.h"
#include "Queue.h"//needed for Level-Order display

class Tree {
public:
    //big three
    Tree();//constructor
    Tree(const Tree& orig);//copy constructor (not defined)
    virtual ~Tree();//virtual destructor (not defined)
    
    //public utilities
    //insert a new node with key (ignores duplicates)
    void insert(std::string key);
    //rebalance tree
    void rebalance();
    //see if such-and-such a string is in the tree
    bool found(std::string findMe);
    //set/cancel Level-Order print mode
    void setLevelOrder(bool setting);
    //set/cancel using demonstration versions of functions, when available
    void setDemoMode(bool setting);
    //print the entire tree (In-Order default, Level-Order available with flag)
    friend std::ostream& operator<<(std::ostream& output, Tree& tree);
    //fast integer log2, needed for vine-to-tree
    unsigned int lg (unsigned int val);
private:
    TreeNode* _root;//the starting point of the tree
    TreeNode* _pseudoRoot;//makes balancing easier
    bool _levelOrder;//if true, tree prints itself in Level-Order, not In-Order
    bool _demoMode;//if true, use "Demo" versions of functions
    int _size;//number of nodes, needed for vine-to-tree
    int _leafCount;//counter of leaves, also needed for vine-to-tree
    
    //internal functions
    //recursive insert
    void plop(TreeNode* node, std::string key);    
    //recursive lookup
    bool lookup(TreeNode* node, std::string findMe);    
    //recursive In-Order traversal
    void inOrder(TreeNode* node, std::ostream& output);
    //iterative Level-Order traversal
    void levelOrder(std::ostream& output);
    //"flattens" tree to one long vine (right children only), allowing for...
    void treeToVine();
    //... conversion back to (balanced!) tree
    void vineToTree();
    //same, with stage-by-stage demo
    void vineToTreeDemo();
    //reposition subtrees to the left
    void compression(int count);    
    //Level-Order traversal just to fix "parentage" for display
    void fixParentage();
};

#endif	/* TREE_H */
